
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <gst/gst.h>
#include <gst/vulkan/vulkan_fwd.h>
G_BEGIN_DECLS

/* enumerations from "gstvkbarrier.h" */
GST_VULKAN_API
GType gst_vulkan_barrier_type_get_type (void);
#define GST_TYPE_VULKAN_BARRIER_TYPE (gst_vulkan_barrier_type_get_type())
GST_VULKAN_API
GType gst_vulkan_barrier_flags_get_type (void);
#define GST_TYPE_VULKAN_BARRIER_FLAGS (gst_vulkan_barrier_flags_get_type())

/* enumerations from "gstvkdisplay.h" */
GST_VULKAN_API
GType gst_vulkan_display_type_get_type (void);
#define GST_TYPE_VULKAN_DISPLAY_TYPE (gst_vulkan_display_type_get_type())

/* enumerations from "gstvkerror.h" */
GST_VULKAN_API
GType gst_vulkan_error_get_type (void);
#define GST_TYPE_VULKAN_ERROR (gst_vulkan_error_get_type())

/* enumerations from "gstvkformat.h" */
GST_VULKAN_API
GType gst_vulkan_format_scaling_get_type (void);
#define GST_TYPE_VULKAN_FORMAT_SCALING (gst_vulkan_format_scaling_get_type())
GST_VULKAN_API
GType gst_vulkan_format_flags_get_type (void);
#define GST_TYPE_VULKAN_FORMAT_FLAGS (gst_vulkan_format_flags_get_type())

/* enumerations from "gstvkhandle.h" */
GST_VULKAN_API
GType gst_vulkan_handle_type_get_type (void);
#define GST_TYPE_VULKAN_HANDLE_TYPE (gst_vulkan_handle_type_get_type())

/* enumerations from "gstvkwindow.h" */
GST_VULKAN_API
GType gst_vulkan_window_error_get_type (void);
#define GST_TYPE_VULKAN_WINDOW_ERROR (gst_vulkan_window_error_get_type())
G_END_DECLS

/* Generated data ends here */

